package MakeWorld::Options;

use strict;
use warnings;

use Moo;

use Getopt::Long qw[ GetOptionsFromArray ];
use Getopt::ArgvFile qw[argvFile];

use File::Path;

use MakeWorld::Log;

use File::Spec::Functions qw/ catfile catdir rel2abs /;

# maybe use MooX::Options in the future?

has builddir   => ( is => 'ro',
		    lazy => 1,
		    default => sub { '.' },
		    coerce  => sub { rel2abs( $_[0] ) },
		  );

has cache      => ( is => 'ro',
		    predicate => 'has_cache'
		  );

has clean      => ( is => 'ro',
		    default => sub { 1 }
		  );

has create_tld => ( is => 'ro',
		    default => sub { 0 }
		  );

has dryrun     => ( is => 'ro',
		    default => sub { 0 }
		  );

has define     => ( is => 'ro',
		    default => sub { {} }
		  );

has odefine    => ( is => 'ro',
		    default => sub { {} }
		  );

has exclude    => ( is => 'ro',
		    lazy => 1,
		    default => sub { {} },

		    # change excluded packages list to a hash to make
		    # life easier split on commas while we're at it
		    coerce  => sub {
			return $_[0] if 'HASH' eq ref $_[0];
			return { map { $_ => 1 }
				 map { split( /[,]/, $_ ) }
				 @{$_[0]}
			       }
		    },
		  );

has exec_prefix => ( is => 'ro',
		     lazy => 1,
		     default => sub { $_[0]->prefix  },
		     coerce => sub { rel2abs( $_[0]) },
		   );

has force      => ( is => 'ro',
		    default => sub { 0 }
		  );

has help       => ( is => 'ro',
		    default => sub { 0 }
		  );

has logdir     => ( is => 'ro',
		    lazy => 1,
		    default => sub { $_[0]->builddir },
		    coerce  => sub { rel2abs( $_[0] ) },
		  );

has logredir   => ( is => 'ro',
		    default => sub { 0 }
		  );

has make_cmd   => ( is => 'ro',
		    default => sub { 'make'}
		  );

has nopkgs     => ( is => 'ro',
		    default => sub { 0 }
		  );

has only       => ( is => 'ro',
		    default => sub { 0 }
		  );

has packages   => ( is => 'ro',
		    default => sub { [] }
		  );

has patch_cmd  => ( is => 'ro',
		    default => sub { 'patch'}
		  );

has pkgs       => ( is => 'ro',
		    lazy => 1,
		    default => sub { [] }
		  );

has prefix     => ( is => 'ro',
		    lazy => 1,
		    coerce => sub { rel2abs( $_[0] ) },
		    predicate => 'has_prefix',
		  );

has srcdir     => ( is => 'ro',
		    lazy => 1,
		    default => sub { [] },
		    coerce => sub { [ map { rel2abs($_)} @{$_[0]} ] },
		  );

has stages     => ( is => 'ro',
		    default => sub { [] },
		    lazy => 1,
		    coerce => sub{ 
			return $_[0] if 'ARRAY' eq ref $_[0];
			return [ split(',',$_[0]) ];
		    },
		  );

has statusfile => ( is => 'ro',
		    lazy => 1,
		    default => sub { catfile( $_[0]->builddir,
					      'make-world.status' ) },
		    coerce => sub { rel2abs( $_[0] ) },
		  );

has usage      => ( is => 'ro',
		    default => sub { 0 }
		  );

has verbose    => ( is => 'ro',
		    lazy => 1,
		    default => sub { 0 },
		    coerce  => sub { $_[0] || 1 }
		  );

has version    => ( is => 'ro',
		    default => sub { 0 }
		  );

has defoption  => ( is => 'ro',
			default => sub { {} }
		      );

has option     => ( is => 'ro',
			 default => sub { {} }
		       );

has PERL5LIB   => ( is => 'ro',
		    lazy => 1,
		    default => sub { [ catdir( $_[0]->prefix, qw{ lib perl5 } ) ] }
		  );

sub BUILDARGS {

    my $self = shift;

    my $argv = @_ ? [ @_ ] : \@ARGV;

    argvFile( resolveEnvVars => 1) ;

    my %opt;

    eval {
	local $SIG{ __WARN__ } = sub { die $_[0] };

	GetOptionsFromArray( $argv, \%opt,
		    qw{
			builddir=s
			cache=s
			clean!
			create_tld
			dryrun|n+
			exclude=s@
			exec_prefix=s
			force!
			help
			logredir!
			logdir=s
			make_cmd=s
			nopkgs
			only
			patch_cmd=s
			pkgs=s@
			prefix=s
			srcdir=s@
			stages=s
			statusfile=s
			usage
			verbose:i
			version
			define|D=s%
			odefine=s%
			defoption=s%
			option=s%
		  }
		  );
    };

    die $@ if $@;

    $opt{packages} = [ @ARGV ];

    return \%opt;
}

sub BUILD {

    my $self = shift;

    MakeWorld::Log::level($self->verbose);

    return if $self->version || $self->help || $self->usage;

    die( "--srcdir not specified\n" )
      unless @{$self->srcdir};

    die( "--prefix not specified\n" )
      unless $self->has_prefix;

    for my $dir ( grep { ! -d $_ } $self->logdir, $self->builddir )
    {
	msg( MakeWorld::Log::Info, "$dir doesn't exist: creating it");
	mkpath( $dir, 0, 0755 );
    }

    unless ( -f $self->statusfile )
    {
	msg( MakeWorld::Log::Info,
	     $self->statusfile . " doesn't exist: creating it");

	MakeWorld::Status::save( $self->statusfile, {} );
    }
}

1;
